@@ -22,12 +22,13 @@
 #ifndef _CALLBACKS_C_
 #define _CALLBACKS_C_
 
-#include <callbacks.h>
-#include <interface.h>
-#include <asciiselect.h>
 #include <config.h>
+#include "callbacks.h"
 #include <unistd.h>
 
+#include "interface.h"
+#include "asciiselect.h"
+
 void
 cb_about_click (GtkWidget *widget, gpointer user_data)
 {
@@ -73,7 +74,7 @@ cb_charbtn_enter (GtkButton *button, gpointer user_data)
     gchar *text, *s;
 
     gtk_label_get (label, &text);
-    s = g_strdup_printf (_("%s: ASCII code %d"), text, (unsigned char) text[0]);
+    s = g_strdup_printf (_("%s: Character code %d"), text, (unsigned char) text[0]);
     gnome_appbar_set_status (GNOME_APPBAR (GNOME_APP (mainapp->window)->statusbar), s);
     gtk_label_set_text (GTK_LABEL (mainapp->preview_label), text);
     g_free (s);
@@ -142,7 +143,7 @@ cb_help_click (GtkWidget *widget, gpointer user_data)
     GnomeHelpMenuEntry *ref;
 
     ref = (GnomeHelpMenuEntry *) g_new0 (GnomeHelpMenuEntry, 1);
-    ref->name = PACKAGE;
+    ref->name = "gcharmap";
     ref->path = "index.html";
     gnome_help_display (NULL, ref);
     g_free (ref);
@@ -239,4 +240,4 @@ cb_toggle_statusbar (GtkCheckMenuItem *checkmenuitem, gpointer user_data)
 }
 
 
-#endif _CALLBACKS_C_
+#endif /* _CALLBACKS_C_ */
