@@ -1,8 +1,8 @@
+#include <config.h>
 #include "logview.h"
 #include <string.h>
 #include <stdlib.h>
 #include <regex.h>
-#include <config.h>
 #include <gnome.h>
 
 #define MAX_NUM_MATCHES     10
@@ -10,17 +10,17 @@
 
 void mon_edit_actions (GtkWidget *widget, gpointer data);
 
+static GList *copy_actions_db (GList *db);
 void free_actions_db (GList **db);
 void free_action (Action *action);
 void print_actions_db (GList *db);
-void close_actions_options (GtkWidget * widget, gpointer client_data);
 void make_tree_from_actions_db (GList *db);
 void clear_actions_db (GList *db);
 
-void edit_actions_entry_cb (GtkWidget *widget, gpointer data);
-void add_actions_entry_cb (GtkWidget *widget, gpointer data);
-void edit_action_entry (Action *action);
-void close_action_record (GtkWidget * widget, GtkWidget *record_widget);
+static void edit_actions_entry_cb (GtkWidget *widget, gpointer data);
+static void remove_actions_entry_cb (GtkWidget *widget, gpointer data);
+static void add_actions_entry_cb (GtkWidget *widget, gpointer data);
+static void edit_action_entry (Action *action);
 
 int exec_action_in_db (Log *log, LogLine *line, GList *db);
 int read_actions_db (char *filename, GList **db);
@@ -32,12 +32,47 @@ extern ConfigData *cfg;
 extern GList *actions_db;
 
 
+static GList *local_actions_db = NULL;
 
-GtkWidget *actions_dialog;
-GtkWidget *actions_edit_dialog;
-GtkCTree *ctree;
-GList *ctree_parent;
+GtkWidget *actions_dialog = NULL;
+GtkCTree *ctree = NULL;
+GtkCTreeNode *selected_node = NULL;
+GList *ctree_parent = NULL;
 
+static void
+tree_select_row (GtkCTree     *ctree,
+		 GtkCTreeNode *row,
+		 gint          column)
+{
+	selected_node = row;
+}
+
+static void
+tree_unselect_row (GtkCTree     *ctree,
+		   GtkCTreeNode *row,
+		   gint          column)
+{
+	if (selected_node == row)
+		selected_node = NULL;
+}
+
+
+static void
+apply_actions (GtkWidget *w, gpointer data)
+{
+	char *fname;
+
+	free_actions_db (&actions_db);
+	actions_db = local_actions_db;
+	local_actions_db = NULL;
+
+	fname = g_strdup_printf ("%s/.gnome/logview-actions.db",
+				 g_get_home_dir ());
+	if (write_actions_db (fname, actions_db)) {
+		gtk_widget_destroy (actions_dialog);
+	}
+	g_free (fname);
+}
 
 
 /* ----------------------------------------------------------------------
@@ -62,11 +97,8 @@ mon_edit_actions (GtkWidget *widget, gpointer data)
   gtk_widget_set_style (actions_dialog, cfg->main_style);
   gtk_widget_set_usize (actions_dialog, 400, -1);
   gtk_signal_connect (GTK_OBJECT (actions_dialog), "destroy",
-		      GTK_SIGNAL_FUNC (close_actions_options),
+		      GTK_SIGNAL_FUNC (gtk_widget_destroyed),
 		      &actions_dialog);
-  gtk_signal_connect (GTK_OBJECT (actions_dialog), "delete_event",
-		      GTK_SIGNAL_FUNC (close_actions_options),
-		      NULL);
   
   vbox = (GtkBox *)gtk_vbox_new (FALSE, 2);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
@@ -76,6 +108,11 @@ mon_edit_actions (GtkWidget *widget, gpointer data)
   /* List with actions */
 
   ctree = GTK_CTREE (gtk_ctree_new_with_titles (1, 0, (char **)title));
+  gtk_signal_connect (GTK_OBJECT (ctree), "tree_select_row",
+		      GTK_SIGNAL_FUNC (tree_select_row), NULL);
+  gtk_signal_connect (GTK_OBJECT (ctree), "tree_unselect_row",
+		      GTK_SIGNAL_FUNC (tree_unselect_row), NULL);
+  selected_node = NULL;
   swin = gtk_scrolled_window_new (NULL, NULL);
   gtk_container_add (GTK_CONTAINER (swin), GTK_WIDGET (ctree));
   gtk_ctree_set_line_style (ctree, GTK_CTREE_LINES_DOTTED);
@@ -111,22 +148,21 @@ mon_edit_actions (GtkWidget *widget, gpointer data)
   gtk_widget_show (hbox);
 
   button = gtk_button_new_with_label (_("Add..."));
-  gtk_widget_set_usize (button, 80, 25);
   gtk_signal_connect (GTK_OBJECT (button), "clicked", 
 		      GTK_SIGNAL_FUNC (add_actions_entry_cb), NULL);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
   gtk_widget_show (button);
   
   button = gtk_button_new_with_label (_("Edit..."));
-  gtk_widget_set_usize (button, 80, 25);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (button), "clicked", 
 		      GTK_SIGNAL_FUNC (edit_actions_entry_cb), NULL);
   gtk_widget_show (button);
   
   button = gtk_button_new_with_label (_("Remove"));
-  gtk_widget_set_usize (button, 80, 25);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked", 
+		      GTK_SIGNAL_FUNC (remove_actions_entry_cb), NULL);
   gtk_widget_show (button);
 
   hbox = gtk_hbox_new (FALSE, 2);
@@ -138,18 +174,26 @@ mon_edit_actions (GtkWidget *widget, gpointer data)
   gtk_widget_show (padding);
   gtk_box_pack_start (GTK_BOX (hbox), padding, TRUE, TRUE, 0);
 
-  button = gtk_button_new_with_label (_("Cancel"));
-  gtk_widget_set_usize (button, 80, 25);
+  button = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
+			     GTK_OBJECT (actions_dialog));
   gtk_widget_show (button);
 
-  button = gtk_button_new_with_label (_("OK"));
-  gtk_widget_set_usize (button, 80, 25);
+  button = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      GTK_SIGNAL_FUNC (apply_actions),
+		      NULL);
   gtk_widget_show (button);
 
+  /* Copy all actions */
+  free_actions_db (&local_actions_db);
+  local_actions_db = copy_actions_db (actions_db);
+
   /* Generate tree with action database */
-  make_tree_from_actions_db (actions_db);
+  make_tree_from_actions_db (local_actions_db);
 
   gtk_widget_show (actions_dialog);
 }
@@ -172,28 +216,24 @@ make_tree_from_actions_db (GList *db)
   text[0] = buffer;
   text[1] = NULL;
   
-  /* Empty list if its not empty */
-  if (ctree_parent)
-    clear_actions_db (db);
-
   /* Create first item */
   ctree_parent = NULL;
   buffer[0] = '\0';
 
-  for(item = g_list_first (db); item != NULL; item=item->next)
+  for(item = db; item != NULL; item=item->next)
     {
       action = (Action *)item->data;
-      if (strncmp (buffer, action->tag, 49) == 0)
-	sibling = (GList *)gtk_ctree_insert_node (ctree, (GtkCTreeNode *)sibling, NULL, (char **)text, 5, NULL, NULL,
-				    NULL, NULL, FALSE, FALSE);
-      else
-	{
+      if (strncmp (buffer, action->tag, 49) != 0) {
 	  strncpy (buffer, action->tag, 50);
-	  sibling = (GList *)gtk_ctree_insert_node (ctree, (GtkCTreeNode *)ctree_parent, NULL, (char **)text, 5, NULL, NULL,
+	  buffer[49] = '\0';
+	  /* this is the parent node */
+	  sibling = (GList *)gtk_ctree_insert_node (ctree, NULL, NULL, (char **)text, 5, NULL, NULL,
 				      NULL, NULL, FALSE, FALSE);
-	  if (ctree_parent == NULL)
-	    ctree_parent = sibling;
+	  ctree_parent = sibling;
 	}
+      /* this is the actual node */
+      sibling = (GList *)gtk_ctree_insert_node (ctree, (GtkCTreeNode *)ctree_parent, NULL, (char **)text, 5, NULL, NULL,
+						NULL, NULL, FALSE, FALSE);
       /* Store data in tree */
       gtk_ctree_node_set_row_data (ctree, (GtkCTreeNode *)sibling,
 				   (gpointer) action);
@@ -204,22 +244,19 @@ make_tree_from_actions_db (GList *db)
 
 }
 
-
-/* ----------------------------------------------------------------------
-   NAME:          close_actions_options
-   DESCRIPTION:   Callback called when the log dialog is closed.
-   ---------------------------------------------------------------------- */
-
-void
-close_actions_options (GtkWidget * widget, gpointer client_data)
+static int
+action_compare (gconstpointer a, gconstpointer b)
 {
-  gtk_widget_hide (actions_dialog);
+	const Action *aa = a;
+	const Action *bb = b;
+
+	return strcmp (aa->tag, bb->tag);
 }
 
 
 
 /* ----------------------------------------------------------------------
-   NAME:        read_regexp_db
+   NAME:        read_action_db
    DESCRIPTION: Reads the database with regular expressions to match.
    ---------------------------------------------------------------------- */
 
@@ -236,10 +273,11 @@ read_actions_db (char *filename, GList **db)
   fp = fopen (filename, "r");
   if (fp == NULL)
     {
-      sprintf (buffer, "Cannot open action data base <%s>! Open failed.", 
-	       filename);
-     ShowErrMessage (buffer);
-     return(-1);
+      g_snprintf (buffer, sizeof (buffer),
+		  _("Cannot open action data base <%s>! Open failed."), 
+		  filename);
+      ShowErrMessage (buffer);
+      return(-1);
     }
 
 
@@ -255,25 +293,17 @@ read_actions_db (char *filename, GList **db)
 	  done = TRUE;
 	  continue;
 	}
-      /* Ignore lines that begin with '#' */
-      if (buffer[0] == '#')
-	continue;
-
       /* Skip spaces */
       c1 = buffer;
       while (*c1 == ' ' || *c1 == '\t') c1++;
       if (*c1 == '\0' || *c1 == '\n')
 	continue; /* Nothing to do here */
+      /* Ignore lines that begin with '#' */
+      if (*c1 == '#')
+	continue;
 
       /* Alloc memory for item */
-      item = malloc (sizeof (Action));
-      if (item == NULL)
-	{
-	  ShowErrMessage ("Error parsing action data base. Out of memory.");
-	  exit (-1);
-	}
-      memset (item, 0, sizeof (Action));
-      
+      item = g_new0 (Action, 1);
 
       /* Read TAG */
       tok = strtok (c1, DELIM);
@@ -347,6 +377,8 @@ read_actions_db (char *filename, GList **db)
 	*db = g_list_append (*db, item);
     }
 
+  *db = g_list_sort (*db, action_compare);
+
   return TRUE;
 }
 
@@ -369,7 +401,7 @@ write_actions_db (char *filename, GList *db)
   if (fp == NULL)
     {
       ShowErrMessage (_("Can't write to actions database!"));
-      return -1;
+      return FALSE;
     }
   
   /* Write a  header to the DB file */
@@ -395,6 +427,8 @@ write_actions_db (char *filename, GList *db)
 	       action->action);
     }
 
+  fclose (fp);
+
   return TRUE;
 }
 
@@ -406,11 +440,11 @@ write_actions_db (char *filename, GList *db)
 void
 free_actions_db (GList **db)
 {
-  clear_actions_db (*db);
-  g_list_free (*db);
-  *db = NULL;
-
-  return;
+	if (*db != NULL) {
+		clear_actions_db (*db);
+		g_list_free (*db);
+		*db = NULL;
+	}
 }
 
 /* ----------------------------------------------------------------------
@@ -422,14 +456,13 @@ int
 exec_action_in_db (Log *log, LogLine *line, GList *db)
 {
   GList *item;
-  Action *cur_action;
+  Action *cur_action = NULL;
   regex_t preg;
   regmatch_t matches[MAX_NUM_MATCHES];
   int doesnt_match;
 
   /* Search for daemon in our list */
-  item = g_list_first (db);
-  for (item = g_list_first (db); item != NULL; item=item->next)
+  for (item = db; item != NULL; item = item->next)
     {
       doesnt_match = FALSE;
       cur_action = (Action *)item->data;
@@ -467,14 +500,14 @@ exec_action_in_db (Log *log, LogLine *line, GList *db)
    DESCRIPTION: Add an entry to the actions DB.
    ---------------------------------------------------------------------- */
 
-void
+static void
 add_actions_entry_cb (GtkWidget *widget, gpointer data)
 {
   Action *action;
 
-  action = malloc (sizeof (Action));
+  action = g_new (Action, 1);
   
-  sprintf (action->tag, _("<empty>"));
+  g_snprintf (action->tag, sizeof (action->tag), _("<empty>"));
   action->log_name = g_strdup (_("log. name regexp"));
   action->process = g_strdup (_("process regexp"));
   action->message = g_strdup (_("message regexp"));
@@ -485,25 +518,94 @@ add_actions_entry_cb (GtkWidget *widget, gpointer data)
 
 }
 
+static void
+remove_actions_entry_cb (GtkWidget *widget, gpointer data)
+{
+  Action *action;
+
+  if (selected_node == NULL)
+	  return;
+
+  action = (Action *) gtk_ctree_node_get_row_data (GTK_CTREE (ctree),
+						   selected_node);
+
+  if (action != NULL) {
+	  local_actions_db = g_list_remove (local_actions_db, action);
+	  gtk_clist_clear (GTK_CLIST (ctree));
+	  selected_node = NULL;
+	  make_tree_from_actions_db (local_actions_db);
+  }
+}
+
 /* ----------------------------------------------------------------------
    NAME:        edit_actions_entry_cb
    DESCRIPTION: Function called when the edit button is pressed.
    ---------------------------------------------------------------------- */
 
-void
+static void
 edit_actions_entry_cb (GtkWidget *widget, gpointer data)
 {
-  GList *item;
   Action *action;
 
-  item = GTK_CLIST (ctree)->selection;
-  if (item == NULL)
-    return;
+  if (selected_node == NULL)
+	  return;
 
-  action = (Action *) gtk_ctree_node_get_row_data(ctree, (GtkCTreeNode *)item);
+  action = (Action *) gtk_ctree_node_get_row_data (GTK_CTREE (ctree),
+						   selected_node);
 
   edit_action_entry (action);
+}
 
+static Action *edited_action = NULL;
+
+static void
+apply_edit (GtkWidget *w, gpointer data)
+{
+	char *text;
+	GtkWidget *action_record = data;
+	GtkWidget *tag =
+		gtk_object_get_data (GTK_OBJECT (action_record), "tag");
+	GtkWidget *log_name =
+		gtk_object_get_data (GTK_OBJECT (action_record), "log_name");
+	GtkWidget *process =
+		gtk_object_get_data (GTK_OBJECT (action_record), "process");
+	GtkWidget *message =
+		gtk_object_get_data (GTK_OBJECT (action_record), "message");
+	GtkWidget *action =
+		gtk_object_get_data (GTK_OBJECT (action_record), "action");
+	GtkWidget *description =
+		gtk_object_get_data (GTK_OBJECT (action_record), "description");
+
+	text = gtk_editable_get_chars (GTK_EDITABLE (tag), 0, -1);
+	strncpy (edited_action->tag, text, 50);
+	edited_action->tag[49] = '\0';
+
+	g_free (edited_action->log_name);
+	edited_action->log_name =
+		gtk_editable_get_chars (GTK_EDITABLE (log_name), 0, -1);
+	g_free (edited_action->process);
+	edited_action->process =
+		gtk_editable_get_chars (GTK_EDITABLE (process), 0, -1);
+	g_free (edited_action->message);
+	edited_action->message =
+		gtk_editable_get_chars (GTK_EDITABLE (message), 0, -1);
+	g_free (edited_action->action);
+	edited_action->action =
+		gtk_editable_get_chars (GTK_EDITABLE (action), 0, -1);
+	g_free (edited_action->description);
+	edited_action->description =
+		gtk_editable_get_chars (GTK_EDITABLE (description), 0, -1);
+
+	if ( ! g_list_find (local_actions_db, edited_action))
+		local_actions_db = g_list_prepend (local_actions_db,
+						   edited_action);
+
+	local_actions_db = g_list_sort (local_actions_db,
+					action_compare);
+
+	gtk_clist_clear (GTK_CLIST (ctree));
+	selected_node = NULL;
+	make_tree_from_actions_db (local_actions_db);
 }
 
 /* ----------------------------------------------------------------------
@@ -511,7 +613,7 @@ edit_actions_entry_cb (GtkWidget *widget, gpointer data)
    DESCRIPTION: Edit a DB record on a window.
    ---------------------------------------------------------------------- */
 
-void
+static void
 edit_action_entry (Action *action)
 {
   GtkWidget *hbox;
@@ -526,19 +628,18 @@ edit_action_entry (Action *action)
 
   if (!action)
     return;
+
+  edited_action = action;
   
   /* Create main window ------------------------------------------------  */
   action_record = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_modal (GTK_WINDOW (action_record), TRUE);
+  gtk_window_set_transient_for (GTK_WINDOW (action_record),
+				GTK_WINDOW (actions_dialog));
   gtk_container_set_border_width (GTK_CONTAINER (action_record), 5);
   gtk_window_set_title (GTK_WINDOW (action_record), "Edit action record");
   gtk_widget_set_style (action_record, cfg->main_style);
   gtk_widget_set_usize (action_record, 400, 400);
-  gtk_signal_connect (GTK_OBJECT (actions_dialog), "destroy",
-		      GTK_SIGNAL_FUNC (close_action_record),
-		      action_record);
-  gtk_signal_connect (GTK_OBJECT (actions_dialog), "delete_event",
-		      GTK_SIGNAL_FUNC (close_action_record),
-		      action_record);
   
   vbox = (GtkBox *)gtk_vbox_new (FALSE, 2);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
@@ -563,6 +664,7 @@ edit_action_entry (Action *action)
   gtk_entry_set_text (GTK_ENTRY (entry), action->tag);
   gtk_tooltips_set_tip (tips, entry, "Tag that identifies the log file.", NULL);
   gtk_widget_show (entry); 
+  gtk_object_set_data (GTK_OBJECT (action_record), "tag", entry);
 
   /* log name */
   hbox = gtk_hbox_new (FALSE, 2);
@@ -580,6 +682,7 @@ edit_action_entry (Action *action)
   gtk_tooltips_set_tip (tips, entry, "Regular expression that will match the log name.", 
 			NULL);
   gtk_widget_show (entry); 
+  gtk_object_set_data (GTK_OBJECT (action_record), "log_name", entry);
       
   /* Process */
   hbox = gtk_hbox_new (FALSE, 2);
@@ -597,6 +700,7 @@ edit_action_entry (Action *action)
   gtk_tooltips_set_tip (tips, entry, "Regular expression that will match process part of message.", 
 			NULL);
   gtk_widget_show (entry); 
+  gtk_object_set_data (GTK_OBJECT (action_record), "process", entry);
 
   /* Message */
   hbox = gtk_hbox_new (FALSE, 2);
@@ -614,6 +718,7 @@ edit_action_entry (Action *action)
   gtk_tooltips_set_tip (tips, entry, "Regular expression that will match the message.", 
 			NULL);
   gtk_widget_show (entry); 
+  gtk_object_set_data (GTK_OBJECT (action_record), "message", entry);
 
   /* Action */
   hbox = gtk_hbox_new (FALSE, 2);
@@ -631,6 +736,7 @@ edit_action_entry (Action *action)
   gtk_tooltips_set_tip (tips, entry, "Action that will be executed if all previous regexps. are matched. This is executed by a system command: system (action).", 
 			NULL);
   gtk_widget_show (entry); 
+  gtk_object_set_data (GTK_OBJECT (action_record), "action", entry);
 
   /* Description */
   hbox = gtk_hbox_new (FALSE, 2);
@@ -644,10 +750,9 @@ edit_action_entry (Action *action)
   gtk_widget_set_usize (text, 200, -1);
   gtk_text_set_editable (GTK_TEXT (text), TRUE);
   gtk_box_pack_start (GTK_BOX (hbox), text, TRUE, TRUE, 0);
-  gtk_entry_set_text (GTK_ENTRY (entry), action->action);
-  gtk_tooltips_set_tip (tips, text, _("Description of this entry."),
-			NULL);
+  gtk_tooltips_set_tip (tips, text, _("Description of this entry."), NULL);
   gtk_widget_show (text); 
+  gtk_object_set_data (GTK_OBJECT (action_record), "description", text);
 
 
   /* Make bottom part ------------------------------------------------ */
@@ -660,37 +765,31 @@ edit_action_entry (Action *action)
   gtk_widget_show (padding);
   gtk_box_pack_start (GTK_BOX (hbox), padding, TRUE, TRUE, 0);
 
-  button = gtk_button_new_with_label (_("Cancel"));
-  gtk_widget_set_usize (button, 80, 25);
+  button = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
+			     GTK_OBJECT (action_record));
   gtk_widget_show (button);
 
-  button = gtk_button_new_with_label (_("OK"));
-  gtk_widget_set_usize (button, 80, 25);
+  button = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
   gtk_widget_show (button);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      GTK_SIGNAL_FUNC (apply_edit),
+		      action_record);
+  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
+			     GTK_OBJECT (action_record));
 
 
   gtk_widget_show (action_record);
-  
 
   /* Insert text into text widget */
   gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, action->description, 
 		   strlen (action->description));
 }
 
-/* ----------------------------------------------------------------------
-   NAME:          close_action_record
-   DESCRIPTION:   Callback called when the log dialog is closed.
-   ---------------------------------------------------------------------- */
-
-void
-close_action_record (GtkWidget * widget, GtkWidget *record_widget)
-{
-  gtk_widget_hide (record_widget);
-}
-
-
 /* ----------------------------------------------------------------------
    NAME:        print_actions_db
    DESCRIPTION: Prints the database (for debbuging purposes).
@@ -737,6 +836,7 @@ clear_actions_db (GList *db)
     {
       action = (Action *)item->data;
       free_action (action);
+      item->data = NULL;
     }
    
   return;
@@ -751,23 +851,42 @@ clear_actions_db (GList *db)
 void
 free_action (Action *action)
 {
-  if (action)
-    {
-      if (!action->log_name)
-	free (action->log_name);
+	if (action) {
+		g_free (action->log_name);
+		g_free (action->process);
+		g_free (action->message);
 
-      if (!action->process)
-	free (action->process);
+		g_free (action->action);
+		g_free (action->description);
 
-      if (!action->message)
-	free (action->message);
+		g_free (action);
+	}
+}
 
-      if (!action->action)
-	free (action->action);
+static GList *
+copy_actions_db (GList *db)
+{
+	GList *li;
+	GList *copy;
 
-      if (!action->description)
-	free (action->description);
-    }
-  
-  return;
+	if (db == NULL)
+		return NULL;
+
+	copy = g_list_copy (db);
+	for (li = copy; li != NULL; li = li->next) {
+		Action *old = li->data;
+		Action *new = g_new0 (Action, 1);
+
+		/* these are the same size so strcpy is safe */
+		strcpy (new->tag, old->tag);
+
+		new->log_name = g_strdup (old->log_name);
+		new->process = g_strdup (old->process);
+		new->message = g_strdup (old->message);
+		new->action = g_strdup (old->action);
+		new->description = g_strdup (old->description);
+		li->data = new;
+	}
+
+	return copy;
 }
