@@ -19,10 +19,11 @@
 
     ---------------------------------------------------------------------- */
 
+#include <config.h>
+#include <gnome.h>
 #include "config.h"
 #include "logview.h"
 #include "logrtns.h"
-#include <gnome.h>
 
 /*
  * -----------------
@@ -56,8 +57,10 @@
 
 int IsLeapYear (int);
 int match_line_in_db (LogLine *line, GList *db);
+void UpdateLastPage (Log *log);
 
 extern GList *regexp_db;
+extern GList *actions_db;
 
 /*
  * -------------------
@@ -66,10 +69,57 @@ extern GList *regexp_db;
  */
 
 char *
-monthname[12] =
-{"January", "February", "March", "April", "May",
- "June", "July", "August", "September", "October",
- "November", "December"};
+C_monthname[12] =
+{ "January",
+  "February",
+  "March",
+  "April",
+  "May",
+  "June",
+  "July",
+  "August",
+  "September",
+  "October",
+  "November",
+  "December" };
+
+static int
+get_month (const char *str)
+{
+	int i;
+	static char *monthname[12] = { 0 };
+
+	for (i = 0; i < 12; i++) {
+		if (g_strncasecmp (str, C_monthname[i], 3) == 0) {
+			return i;
+		}
+	}
+
+	for (i = 0; i < 12; i++) {
+		if (monthname[i] == NULL) {
+			GDate *date;
+			char buf[256];
+
+			date = g_date_new_dmy (1, i+1, 2000 /* bogus */);
+
+			if (g_date_strftime (buf, sizeof (buf), "%b", date) <= 0) {
+				/* eek, just use C locale cuz we're screwed */
+				monthname[i] = g_strndup (C_monthname[i], 3);
+			} else {
+				monthname[i] = g_strdup (buf);
+			}
+
+			g_date_free (date);
+		}
+
+		if (g_strcasecmp (str, monthname[i]) == 0) {
+			return i;
+		}
+	}
+
+	return 12;
+}
+
 
 
 /* ----------------------------------------------------------------------
@@ -122,7 +172,8 @@ OpenLogFile (char *filename)
    first->prev = cur;
 
    /* Open log files ------------------------------------- */
-   strcpy (tlog->name, filename);
+   strncpy (tlog->name, filename, sizeof (tlog->name)-1);
+   tlog->name[sizeof (tlog->name) - 1] = '\0';
    tlog->fp = fopen (filename, "r");
    if (tlog->fp == NULL)
    {
@@ -170,7 +221,7 @@ int
 isLogFile (char *filename)
 {
    struct stat fstatus;
-   char buff[256];
+   char buff[1024];
    char *token;
    int i;
    FILE *fp;
@@ -180,14 +231,16 @@ isLogFile (char *filename)
    /* Check that its a regular file       */
    if (!S_ISREG (fstatus.st_mode))
    {
-      sprintf (buff, _("%s is not a regular file."), filename);
+      g_snprintf (buff, sizeof (buff),
+		  _("%s is not a regular file."), filename);
       ShowErrMessage (buff);
       return FALSE;
    }
    /* File unreadable                     */
    if ((fstatus.st_mode & S_IRUSR) == 0)
    {
-      sprintf (buff, _("%s is not user readable. Probably has read-only "
+      g_snprintf (buff, sizeof (buff),
+		  _("%s is not user readable. Probably has read-only "
       "permission. Either run the program as root or ask the sysadmin to "
       "change the permissions on the file."), filename);
       ShowErrMessage (buff);
@@ -199,26 +252,31 @@ isLogFile (char *filename)
    fp = fopen (filename, "r");
    if (fp == NULL)
    {
-      sprintf (buff, _("%s could not be opened."), filename);
+      g_snprintf (buff, sizeof (buff),
+		  _("%s could not be opened."), filename);
       ShowErrMessage (buff);
       return FALSE;
    }
    fgets (buff, 255, fp);
    fclose (fp);
    token = strtok (buff, " ");
+   /* This is not a good assumption I don't think, especially
+    * if log is internationalized
+    * -George
    if (strlen (buff) != 3)
    {
-      sprintf (buff, _("%s not a log file."), filename);
+      g_snprintf (buff, sizeof (buff),
+		  _("%s not a log file."), filename);
       ShowErrMessage (buff);
       return FALSE;
    }
-   for (i = 0; i < 12; i++)
-      if (strncasecmp (token, monthname[i], 3) == 0)
-	 break;
+   */
+
+   i = get_month (token);
 
    if (i == 12)
    {
-      sprintf (buff, _("%s not a log file."), filename);
+      g_snprintf (buff, sizeof (buff), _("%s not a log file."), filename);
       ShowErrMessage (buff);
       return FALSE;
    }
@@ -273,7 +331,7 @@ ReadNPagesDown (Log * lg, Page * pg, int n)
    /* Read n pages down.   */
    for (i = 0; i < n; i++)
    {
-      ReadPageDown (lg, cp);
+      ReadPageDown (lg, cp, FALSE /* exec_actions */);
       lg->lastpg = cp;
       lg->firstpg = lg->firstpg->next;
       if (cp->islastpage == TRUE)
@@ -361,7 +419,7 @@ ReadPageUp (Log * lg, Page * pg)
    ---------------------------------------------------------------------- */
 
 int
-ReadPageDown (Log * lg, Page * pg)
+ReadPageDown (Log * lg, Page * pg, gboolean exec_actions)
 {
    FILE *fp;
    LogLine *line;
@@ -420,6 +478,10 @@ ReadPageDown (Log * lg, Page * pg)
 
       line = &pg->line[ln];
       ParseLine (buffer, line);
+
+      if (exec_actions)
+	      exec_action_in_db (lg, line, actions_db);
+
       ln++;
    }
    pg->ll = ln-1;
@@ -444,6 +506,9 @@ ParseLine (char *buff, LogLine * line)
 
    token = strtok (buff, " ");
    if (token == NULL) return;
+   /* This is not a good assumption I don't think, especially
+    * if log is internationalized
+    * -George
    if (strlen (token) != 3)
    {
       strncpy (line->message, buff, MAX_WIDTH);
@@ -456,13 +521,13 @@ ParseLine (char *buff, LogLine * line)
       strcpy (line->process, "");
       return;
    }
-   for (i = 0; i < 12; i++)
-      if (strncasecmp (token, monthname[i], 3) == 0)
-	 break;
+   */
+   i = get_month (token);
 
    if (i == 12)
    {
       strncpy (line->message, buff, MAX_WIDTH);
+      line->message[MAX_WIDTH-1] = '\0';
       line->month = -1;
       line->date = -1;
       line->hour = -1;
@@ -529,10 +594,10 @@ ParseLine (char *buff, LogLine * line)
    }
 
    token = strtok (NULL, " ");
-   if (token != NULL)
+   if (token != NULL) {
       strncpy (line->hostname, token, MAX_HOSTNAME_WIDTH);
-   else
-   {
+      line->hostname[MAX_HOSTNAME_WIDTH-1] = '\0';
+   } else {
       line->sec = -1;
       strcpy (line->hostname, "");
       strcpy (line->process, "-");
@@ -545,6 +610,7 @@ ParseLine (char *buff, LogLine * line)
       strncpy (scratch, token, 254);
    else
       strncpy (scratch, " ", 254);
+   scratch[254] = '\0';
    token = strtok (NULL, "\n");
 
    if (token == NULL)
@@ -554,12 +620,15 @@ ParseLine (char *buff, LogLine * line)
       while (scratch[i] == ' ')
 	 i++;
       strncpy (line->message, &scratch[i], MAX_WIDTH);
+      line->message [MAX_WIDTH-1] = '\0';
    } else
    {
       strncpy (line->process, scratch, MAX_PROC_WIDTH);
+      line->process [MAX_PROC_WIDTH-1] = '\0';
       while (*token == ' ')
 	 token++;
       strncpy (line->message, token, MAX_WIDTH);
+      line->message [MAX_WIDTH-1] = '\0';
    }
 
    /* Search current data base for string and attach descritpion */
@@ -850,7 +919,7 @@ MoveToMark (Log *log)
     fseek(fp, mark->offset-1, SEEK_SET);
   else
     fseek(fp, mark->offset, SEEK_SET);
-  ReadPageDown(log, middlepg);
+  ReadPageDown(log, middlepg, FALSE /* exec_actions */);
   if (middlepg->islastpage)
     {
       middlepg = log->lastpg;
@@ -858,7 +927,7 @@ MoveToMark (Log *log)
 	fseek(fp, mark->offset-1, SEEK_SET);
       else
 	fseek(fp, mark->offset, SEEK_SET);
-      ReadPageDown(log, middlepg);
+      ReadPageDown(log, middlepg, FALSE /* exec_actions */);
       ReadNPagesUp(log, middlepg, NUM_PAGES-1);
       log->currentpg = middlepg;
       return;
@@ -868,7 +937,7 @@ MoveToMark (Log *log)
     {
       middlepg = log->firstpg;
       fseek(fp, mark->offset, SEEK_SET);
-      ReadPageDown(log, middlepg);
+      ReadPageDown(log, middlepg, FALSE /* exec_actions */);
       ReadNPagesDown(log, middlepg, NUM_PAGES-1);
       log->currentpg = middlepg;
       return;
@@ -943,9 +1012,7 @@ GetDate (char *line)
    if (!token)
        return -1;
    
-   for (i = 0; i < 12; i++)
-      if (strncasecmp (token, monthname[i], 3) == 0)
-	 break;
+   i = get_month (token);
 
    if (i == 12)
       return -1;
